from typing import TypedDict


class LibraryExecuteArgs(TypedDict):
    year: int
    variant: int


class EventExecuteArgs(TypedDict):
    variant: int


class TaskParameters(object):
    def __init__(self, day: int, year: int, input: str):
        self.day = day
        self.year = year
        self.input = input

        self.variant = None

    def set_variant(self, variant: int):
        self.variant = variant



from __future__ import annotations

from typing import Optional, Type, TYPE_CHECKING
from typing_extensions import Unpack

from .helper import EventExecuteArgs, TaskParameters

if TYPE_CHECKING:
    from aoc import AdventOfCode, AdventOfCodeTask


class AdventOfCodeEvent:
    def __init__(self, instance: AdventOfCode, year: int):
        self.year = year
        self.instance = instance

        self.registered_tasks = {}

    def register_task(self, day: int, task: Type[AdventOfCodeTask]):
        self.registered_tasks[day] = task

    def execute(self, day: int, **kwargs: Unpack[EventExecuteArgs]):
        if self.instance.config.debug:
            print(f"Executing day {day} - year: {self.year}")

        # self.__check_no_tasks()
        assert day in self.registered_tasks, f"Task for {day}. day is not registered!"

        task_input = self.instance.load_input(day, self.year)
        task = self.registered_tasks[day]()

        task.execute(kwargs.get('variant'), TaskParameters(day, self.year, task_input))

    def execute_all(self, **kwargs: Unpack[EventExecuteArgs]):
        self.__check_no_tasks()

        for day in self.registered_tasks.keys():
            self.execute(day, **kwargs)

    def execute_last(self, **kwargs: Unpack[EventExecuteArgs]):
        self.__check_no_tasks()

        last_day = list(self.registered_tasks.keys())[-1]

        self.execute(last_day, **kwargs)

    def __check_no_tasks(self):
        assert self.registered_tasks, f"No tasks added for this event! ({self.year})"



from __future__ import annotations

from os import path, mkdir
from typing import Optional, Type, TYPE_CHECKING
from typing_extensions import Unpack

import requests

from .objects.helper import LibraryExecuteArgs

if TYPE_CHECKING:
    from aoc import AdventOfCodeEvent, AdventOfCodeTask


class AdventOfCodeConfig:
    def __init__(self):
        self.debug = False
        self.testing = False

        self.auto_fetch_input = False
        self.cache_input = False

        self.session: Optional[str] = None

        self.cache_directory: Optional[str] = None


class AdventOfCode:
    def __init__(self, year: Optional[int] = None):
        self.config = AdventOfCodeConfig()

        self.events = {}
        self.last_year = 0
        self.manual_input: Optional[str] = None

        if year:
            self.add_year(year)

    def enable_debug(self):
        self.config.debug = True

    def enable_auto_input_fetch(self, session: str, cache: bool = False, cache_directory: str = "/cache"):
        self.config.session = session
        self.config.auto_fetch_input = True

        if cache:
            self.config.cache_directory = cache_directory
            self.config.cache_input = cache

            if not path.exists(self.config.cache_directory):
                mkdir(self.config.cache_directory)

    def add_year(self, year: int) -> AdventOfCodeEvent:
        self.last_year = year

        from aoc import AdventOfCodeEvent
        event = AdventOfCodeEvent(self, year)
        self.events[year] = event

        return event

    def register_task(self, day: int, task: Type[AdventOfCodeTask]):
        self.events[self.last_year].register_task(day, task)

    def execute(self, day: int, **kwargs: Unpack[LibraryExecuteArgs]):
        event = self.get_event(kwargs.get('year'))
        event.execute(day, **kwargs)

    def execute_all(self, **kwargs: Unpack[LibraryExecuteArgs]):
        event = self.get_event(kwargs.get('year'))
        event.execute_all(**kwargs)

    def execute_last(self, **kwargs: Unpack[LibraryExecuteArgs]):
        event = self.get_event(kwargs.get('year'))
        event.execute_last(**kwargs)

    def load_input(self, day: int, year: Optional[int] = None) -> Optional[str]:
        if self.config.testing:
            return "Dummy"

        if self.manual_input:
            return self.manual_input

        if not year:
            year = self.last_year

        task_input = None
        cache_file = None

        if self.config.cache_input:
            cache_file = path.join(self.config.cache_directory, f"{year}-{day}.tmp")

            if path.exists(cache_file):
                with open(cache_file, "r") as file:
                    task_input = file.read()

        if not task_input:
            task_input = self.fetch_input(day, year)

            if self.config.cache_input:
                with open(cache_file, "w") as file:
                    file.write(task_input)

        return task_input

    def fetch_input(self, day: int, year: Optional[int] = None) -> Optional[str]:
        # if not self.config.auto_fetch_input:
        #    return None

        assert self.config.auto_fetch_input, "Automatic fetching of input is disabled, enable it using 'enable_auto_input_fetch' or set manually input by 'set_input'!"

        if not year:
            year = self.last_year

        url = f"https://adventofcode.com/{year}/day/{day}/input"

        if self.config.debug:
            print(f"Fetching input from {url}")

        response = requests.get(url, cookies={'session': self.config.session})

        if response.status_code != 200:
            print(f"Failed to fetch input, error: {response.text}")

            return None

        return response.text

    def get_event(self, year: Optional[int] = None) -> AdventOfCodeEvent:
        if not year:
            year = self.last_year

        assert year in self.events, f"No event for year {year} is created!"

        return self.events[year]

    def set_input(self, manual_input: str):
        self.manual_input = manual_input



from aoc import AdventOfCode, AdventOfCodeTask

current_day = 0
current_year = 0

executed_tasks = 0


class TestTask(AdventOfCodeTask):
    def run(self):
        if current_day != 0:
            assert current_day == self.task_day, "Wrong day executed!"
        if current_year != 0:
            assert current_year == self.task_year, "Wrong year executed!"

        global executed_tasks
        executed_tasks += 1


def add_year(instance: AdventOfCode, year: int):
    instance.add_year(year)

    for i in range(1, 4):
        instance.register_task(i, TestTask)


def add_year_event(instance: AdventOfCode, year: int):
    event = instance.add_year(year)

    for i in range(1, 4):
        event.register_task(i, TestTask)


def execute_year(instance: AdventOfCode, year: int):
    global current_year
    current_year = year

    global current_day

    for i in range(1, 4):
        current_day = i
        instance.execute(i, year=year)

    global executed_tasks
    assert executed_tasks == 3, "Wrong amount of tasks has been executed!"
    executed_tasks = 0


def test_aoc():
    instance = AdventOfCode()

    # Mark instance as testing to supply dummy input
    instance.config.testing = True

    # Adding by last year
    for i in range(2015, 2018):
        add_year(instance, i)

    # Adding using event instance
    for i in range(2022, 2019, -1):
        add_year_event(instance, i)

    # Executing
    for i in range(2015, 2018):
        execute_year(instance, i)

    for i in range(2022, 2019, -1):
        execute_year(instance, i)



from abc import ABC, abstractmethod
from typing import Type, Optional

from .helper import TaskParameters


class AdventOfCodeTaskVariant(ABC):
    # noinspection PyAttributeOutsideInit
    def setup(self, parameters: TaskParameters):  # __setup
        self.parameters = parameters

    @abstractmethod
    def run(self):
        pass


class AdventOfCodeTask(AdventOfCodeTaskVariant):
    def add_variant(self, index: int, variant: Type[AdventOfCodeTaskVariant]):
        self.__check_index(index)
        self.__check_variants()

        self.variants[index] = variant

    def execute(self, index: Optional[int], parameters: TaskParameters):
        self.__check_index(index)
        self.__check_variants()

        if index is None:
            self.setup(parameters)
            self.run()

            return

        assert index in self.variants, f"No variant for index {index} is added!"

        variant = self.variants[index]()
        parameters.set_variant(index)

        variant.setup(parameters)
        variant.run()

    def run(self):
        pass

    # noinspection PyMethodMayBeStatic
    def __check_index(self, index: int):
        if index is None:
            return

        assert index >= 0, "Variant index cannot be negative!"

    # noinspection PyUnresolvedReferences
    def __check_variants(self):
        if not hasattr(self, 'variants'):
            self.variants = {}



from aoc import AdventOfCode, AdventOfCodeTask, AdventOfCodeTaskVariant


class TestTaskFirstVariant(AdventOfCodeTaskVariant):
    def run(self):
        print(f"First Variant")
        print(f"- Parameters: {self.parameters.__dict__}")


class TestTaskSecondVariant(AdventOfCodeTaskVariant):
    def run(self):
        print(f"Second Variant")
        print(f"- Parameters: {self.parameters.__dict__}")


class TestTask(AdventOfCodeTask):
    def __init__(self):
        self.add_variant(0, TestTaskFirstVariant)
        self.add_variant(1, TestTaskSecondVariant)

    def run(self):
        print(f"Third Variant")
        print(f"- Parameters: {self.parameters.__dict__}")


def main():
    instance = AdventOfCode(2022)
    instance.set_input("Hello, world!")

    instance.register_task(1, TestTask)

    instance.execute(1)


if __name__ == '__main__':
    main()



import setuptools


with open("README.md", "r") as file:
    long_description = file.read()


setuptools.setup(
    name='aoc-library',
    version='0.1.4',
    description='Advent of Code Library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Area110011/AdventOfCodeLibrary/',
    author='Liborsaf, Thr0nSK',
    author_email='liborsaf@wetian.eu',
    license='GPL-v3',
    classifiers=[],
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=['requests', 'typing-extensions'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests',
    python_requires='>=3.11'
)



from aoc import AdventOfCode, AdventOfCodeTask

current_task = 0
executed_tasks = 0


class TestTask(AdventOfCodeTask):
    def run(self):
        if current_task != 0:
            assert current_task == self.task_day, "Wrong day executed!"

        global executed_tasks
        executed_tasks += 1


def test_aoc():
    global current_task

    instance = AdventOfCode(2022)
    instance.register_task(1, TestTask)
    instance.register_task(2, TestTask)
    instance.register_task(3, TestTask)

    # Mark instance as testing to supply dummy input
    instance.config.testing = True

    instance.execute(current_task := 1)
    instance.execute(current_task := 2)
    instance.execute(current_task := 3)

    instance.register_task(4, TestTask)

    current_task = 4
    instance.execute_last()

    global executed_tasks
    assert executed_tasks == 4, "Wrong amount of tasks has been executed!"

    current_task = 0
    executed_tasks = 0

    instance.execute_all()
    assert executed_tasks == 4, "Wrong amount of tasks has been executed!"
